package ru.dbotthepony.mc.otm.entity

import net.minecraft.world.entity.AnimationState
import net.minecraft.world.entity.EntityType
import net.minecraft.world.entity.ai.attributes.AttributeSupplier
import net.minecraft.world.entity.ai.attributes.Attributes
import net.minecraft.world.entity.ai.goal.LeapAtTargetGoal
import net.minecraft.world.entity.ai.goal.LookAtPlayerGoal
import net.minecraft.world.entity.ai.goal.RandomLookAroundGoal
import net.minecraft.world.entity.ai.goal.WaterAvoidingRandomStrollGoal
import net.minecraft.world.entity.ai.goal.target.HurtByTargetGoal
import net.minecraft.world.entity.ai.navigation.GroundPathNavigation
import net.minecraft.world.entity.ai.navigation.PathNavigation
import net.minecraft.world.entity.monster.Monster
import net.minecraft.world.entity.player.Player
import net.minecraft.world.level.Level

class BreadMonster(type: EntityType<BreadMonster>, level: Level) : Monster(type, level) {
	val idleState = AnimationState()

	init {
	    idleState.start(tickCount)
	}

	override fun registerGoals() {
		goalSelector.addGoal(1, LeapAtTargetGoal(this, 0.4f))
		goalSelector.addGoal(2, WaterAvoidingRandomStrollGoal(this, 0.8))
		goalSelector.addGoal(3, LookAtPlayerGoal(this, Player::class.java, 8f))
		goalSelector.addGoal(4, RandomLookAroundGoal(this))

		targetSelector.addGoal(1, HurtByTargetGoal(this))
	}

	override fun createNavigation(level: Level): PathNavigation = GroundPathNavigation(this, level)

	companion object {
		fun createAttributes() : AttributeSupplier.Builder {
			return createMonsterAttributes()
				.add(Attributes.MAX_HEALTH, 16.0)
				.add(Attributes.MOVEMENT_SPEED, 0.3)
		}
	}
}
